# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand

#' Filter DataFrame
#'
#' Filter a DataFrame based on the specified filter list.
#'
#' @param df The DataFrame to be filtered
#' @param filter_list A list specifying the filters to be applied
#' @return The filtered DataFrame
#' @export
#'
#' @examples
#' # Create a DataFrame
#' df <- data.frame(ID = c(1, 2, 3, 4),
#'                  Name = c("John", "Jane", "Mike", "Sarah"),
#'                  Age = c(25, 30, 28, 32),
#'                  Gender = c("Male", "Female", "Male", "Female"))
#'
#' # Create a filter list
#' filter_list <- list(Name = c("John", "Mike"), Age = c(25, 28))
#'
#' # Filter the DataFrame
#' filter_dataframe(df, filter_list)
filter_dataframe <- function(df, filter_list ) {
  filtered_df <- df

  for (attr_name in attributes(filter_list)$names) {
    attr_value <- filter_list[[attr_name]]

    filtered_df <- filtered_df[filtered_df[[attr_name]] %in% attr_value, ]
  }

  return(filtered_df)
}


#' Calculate actual interaction values between two terms in a data frame.
#'
#' This function calculates the actual interaction values between two terms, \code{lbl_term_1} and \code{lbl_term_2},
#' in the given data frame \code{data}. The interaction values are computed based on the mean log expression levels
#' of the conditions satisfying the specified term combinations, and also considering a reference condition.
#'
#' @param data A data frame containing the expression data and associated terms.
#' @param l_reference A data frame representing the reference condition for the interaction.
#' @param clmn_term_1 The name of the column in \code{data} representing the first term.
#' @param lbl_term_1 The label of the first term to compute interactions for.
#' @param clmn_term_2 The name of the column in \code{data} representing the second term.
#' @param lbl_term_2 The label of the second term to compute interactions for.
#'
#' @return A numeric vector containing the actual interaction values between the specified terms.
#' @export
#' @examples
#' average_gt <- data.frame(clmn_term_1 = c("A", "A", "B", "B"), 
#'                          clmn_term_2 = c("X", "Y", "Y", "X"),
#'                          logQij_mean = c(1.5, 8.0, 0.5, 4.0))
#' # Définir les paramètres de la fonction
#' l_label <- list(clmn_term_1 = c("A", "B"), clmn_term_2 = c("X", "Y"))
#' clmn_term_1 <- "clmn_term_1"
#' lbl_term_1 <- "B"
#' clmn_term_2 <- "clmn_term_2"
#' lbl_term_2 <- "Y"
#' # Calculer la valeur d'interaction réelle
#' actual_interaction <- calculate_actual_interactionX2_values(average_gt, 
#'                                        l_label, clmn_term_1, lbl_term_1, 
#'                                        clmn_term_2, lbl_term_2)
calculate_actual_interactionX2_values <- function(data, l_reference , clmn_term_1, lbl_term_1, clmn_term_2, lbl_term_2) {
  A <- data[data[[clmn_term_1]] == lbl_term_1 & 
              data[[clmn_term_2]] == lbl_term_2, ]
  B <- data[data[[clmn_term_1]] == lbl_term_1 & 
              data[[clmn_term_2]] == l_reference[[clmn_term_2]][1], ]
  C <- data[data[[clmn_term_1]] == l_reference[[clmn_term_1]][1] & 
              data[[clmn_term_2]] == lbl_term_2, ]
  D <- data[data[[clmn_term_1]] == l_reference[[clmn_term_1]][1] &
              data[[clmn_term_2]] == l_reference[[clmn_term_2]][1], ]
  actual_interaction <- (A$logQij_mean - B$logQij_mean) - (C$logQij_mean - D$logQij_mean)
  return(actual_interaction)
}


#' Prepare data for computing interaction values.
#'
#' This function prepares the data for computing interaction values between variables.
#' It filters the \code{dataActual} data frame by selecting only the rows where the categorical variables
#' specified in \code{categorical_vars} are at their reference levels.
#'
#' @param categorical_vars A character vector containing the names of categorical variables.
#' @param categorical_varsInInteraction A character vector containing the names of categorical variables involved in interactions.
#' @param dataActual A data frame containing the actual data with categorical variables and associated expression levels.
#'
#' @return A data frame containing the filtered data for computing interaction values.
#' @export
prepareData2computeInteraction <- function(categorical_vars, categorical_varsInInteraction, dataActual){
  l_RefInCategoricalVars <- lapply(dataActual[, categorical_vars], function(vector) levels(vector)[1])
  l_categoricalVars_NOT_InInteraction <-  categorical_vars[! categorical_vars %in% categorical_varsInInteraction ]
  l_filter <- l_RefInCategoricalVars[l_categoricalVars_NOT_InInteraction]
  dataActual_2computeInteractionValues <- filter_dataframe(dataActual, l_filter)
  return(dataActual_2computeInteractionValues)
}



#' Generate actual values for the interaction fixed effect.
#'
#' This function calculates the actual values for the interaction fixed effect
#' based on the input labels in the interaction, categorical variables in the interaction,
#' data to compute interaction values, actual intercept, and the reference levels in
#' categorical variables.
#'
#' @param labelsInInteraction A vector containing the labels of the interaction terms.
#' @param l_categoricalVarsInInteraction A vector containing the names of categorical variables
#'                                        involved in the interaction.
#' @param data2computeInteraction The data frame used to compute interaction values.
#' @param l_RefInCategoricalVars A list containing the reference levels of categorical variables.
#'
#' @return A data frame with the actual values for the interaction fixed effect.
#' The data frame includes columns: term, actual, and description.
#'
#' @export
generateActualInteractionX2_FixEff <- function(labelsInInteraction, l_categoricalVarsInInteraction, 
                                               data2computeInteraction, l_RefInCategoricalVars ){
  clmn_term_1 <- l_categoricalVarsInInteraction[1]
  lbl_term_1 <- labelsInInteraction[1]
  clmn_term_2 <- l_categoricalVarsInInteraction[2]
  lbl_term_2 <- labelsInInteraction[2]
  interactionValues <- calculate_actual_interactionX2_values(data2computeInteraction,
                                                              l_RefInCategoricalVars, clmn_term_1,
                                                              lbl_term_1, clmn_term_2, lbl_term_2)


  df_actualForMyInteraction <- data.frame(geneID = unique(data2computeInteraction$geneID))
  df_actualForMyInteraction$term <- paste(labelsInInteraction, collapse = ":")
  df_actualForMyInteraction$actual <- interactionValues
  df_actualForMyInteraction$description <- paste(gsub("\\d+$", "", lbl_term_1) , 
                                                 gsub("\\d+$", "", lbl_term_2), sep = ":")

  return(df_actualForMyInteraction)

}


#' Generate Actual Interaction Values for Three Fixed Effects
#'
#' This function generates actual interaction values for three fixed effects in a dataset. It takes the labels of the three fixed effects, the dataset, and the reference values for the categorical variables. The function computes the actual interaction values and returns a data frame containing the geneID, the term description, and the actual interaction values.
#'
#' @param labelsInInteraction A character vector of labels for the three fixed effects.
#' @param l_categoricalVarsInInteraction A list of categorical variable names corresponding to the three fixed effects.
#' @param data2computeInteraction The dataset on which to compute the interaction values.
#' @param l_RefInCategoricalVars A list of reference values for the categorical variables.
#'
#' @return A data frame with geneID, term description, and actual interaction values.
#'
#' @export
generateActualInteractionX3_FixEff <- function(labelsInInteraction, l_categoricalVarsInInteraction,
                                            data2computeInteraction, l_RefInCategoricalVars) {

   clmn_term_1 <- l_categoricalVarsInInteraction[1]
  lbl_term_1 <- labelsInInteraction[1]
  clmn_term_2 <- l_categoricalVarsInInteraction[2]
  lbl_term_2 <- labelsInInteraction[2]
  clmn_term_3 <- l_categoricalVarsInInteraction[3]
  lbl_term_3 <- labelsInInteraction[3]
  interactionValues <- calculate_actual_interactionX3_values(data2computeInteraction,
                                                          l_RefInCategoricalVars, clmn_term_1,
                                                           lbl_term_1, clmn_term_2, lbl_term_2, lbl_term_3, clmn_term_3)


  df_actualForMyInteraction <- data.frame(geneID = unique(data2computeInteraction$geneID))
  df_actualForMyInteraction$term <- paste(labelsInInteraction, collapse = ":")
  df_actualForMyInteraction$actual <- interactionValues
  df_actualForMyInteraction$description <- paste(gsub("\\d+$", "", lbl_term_1) ,
                                                 gsub("\\d+$", "", lbl_term_2),
                                                 gsub("\\d+$", "", lbl_term_3), sep = ":")

  return(df_actualForMyInteraction)
  
}


#' Calculate Actual Interaction Values for Three Fixed Effects
#'
#' This function calculates actual interaction values for three fixed effects in a dataset. It takes the data, reference values for categorical variables, and the specifications for the fixed effects. The function computes the interaction values and returns the result.
#'
#' @param data The dataset on which to calculate interaction values.
#' @param l_reference A list of reference values for categorical variables.
#' @param clmn_term_1 The name of the first categorical variable.
#' @param lbl_term_1 The label for the first categorical variable.
#' @param clmn_term_2 The name of the second categorical variable.
#' @param lbl_term_2 The label for the second categorical variable.
#' @param lbl_term_3 The label for the third categorical variable.
#' @param clmn_term_3 The name of the third categorical variable.
#'
#' @return The computed actual interaction values.
#'
#' @export
calculate_actual_interactionX3_values <- function(data, l_reference, clmn_term_1, lbl_term_1, 
                                                  clmn_term_2, lbl_term_2, lbl_term_3, clmn_term_3) {
  ## Label term 3
  A <- data[data[[clmn_term_1]] == lbl_term_1 & 
              data[[clmn_term_2]] == lbl_term_2 & 
              data[[clmn_term_3]] == lbl_term_3, ]
  
  B <- data[data[[clmn_term_1]] == l_reference[[clmn_term_1]][1] & 
              data[[clmn_term_2]] == lbl_term_2 & 
              data[[clmn_term_3]] == lbl_term_3 , ]
  
  C <- data[data[[clmn_term_1]] == lbl_term_1 & 
              data[[clmn_term_2]] == l_reference[[clmn_term_2]][1] & 
              data[[clmn_term_3]] == lbl_term_3, ]
  
  D <- data[data[[clmn_term_1]] == l_reference[[clmn_term_1]][1] & 
              data[[clmn_term_2]] == l_reference[[clmn_term_2]][1] & 
              data[[clmn_term_3]] == lbl_term_3, ]
  
  termA = (A$logQij_mean-B$logQij_mean) - (C$logQij_mean - D$logQij_mean)
  
  ## Label term 3 == reference !
  A <- data[data[[clmn_term_1]] == lbl_term_1 & 
              data[[clmn_term_2]] == lbl_term_2 & 
              data[[clmn_term_3]] == l_reference[[clmn_term_3]][1], ]
  
  B <- data[data[[clmn_term_1]] == l_reference[[clmn_term_1]][1] & 
              data[[clmn_term_2]] == lbl_term_2 & 
              data[[clmn_term_3]] == l_reference[[clmn_term_3]][1] , ]
  
  C <- data[data[[clmn_term_1]] == lbl_term_1 & 
              data[[clmn_term_2]] == l_reference[[clmn_term_2]][1] & 
              data[[clmn_term_3]] == l_reference[[clmn_term_3]][1], ]
  
  D <- data[data[[clmn_term_1]] == l_reference[[clmn_term_1]][1] & 
              data[[clmn_term_2]] == l_reference[[clmn_term_2]][1] & 
              data[[clmn_term_3]] == l_reference[[clmn_term_3]][1], ]
  
  termB = (A$logQij_mean-B$logQij_mean) - (C$logQij_mean - D$logQij_mean)
  actual_interaction <- termA - termB
  return(actual_interaction)
}



#' Get the actual interaction values for a given interaction term in the data.
#'
#' This function takes an interaction term, the dataset, and the names of the categorical variables 
#' as inputs. It calculates the actual interaction values based on the difference in log-transformed 
#' mean expression levels for the specified interaction term. The function first prepares the data for 
#' computing the interaction values and then generates the actual interaction values.
#'
#' @param labelsInInteraction A character vector containing the labels of the categorical levels 
#'     involved in the interaction.
#' @param data The dataset containing the gene expression data and categorical variables.
#' @param categorical_vars A character vector containing the names of the categorical variables in 
#'     the dataset.
#' @return A data frame containing the actual interaction values.
#' @export 
getActualInteractionFixEff <- function(labelsInInteraction, data, categorical_vars ){
  l_RefInCategoricalVars <- lapply(data[, categorical_vars], function(vector) levels(vector)[1])
  l_labelsInCategoricalVars <- lapply(data[, categorical_vars], levels)
  l_categoricalVarsInInteraction <- lapply(labelsInInteraction,
                                           function(label) findAttribute(label, 
                                                        l_labelsInCategoricalVars)) %>% 
                                    unlist()
  data2computeInteraction <- prepareData2computeInteraction(categorical_vars, l_categoricalVarsInInteraction,  data )

  ## Interaction x3
  if (length(labelsInInteraction) == 3){
        actualInteractionValues <- generateActualInteractionX3_FixEff(labelsInInteraction,
                                                                     l_categoricalVarsInInteraction ,
                                                                     data2computeInteraction, 
                                                                     l_RefInCategoricalVars)
  }
  # Interaction x2
  if (length(labelsInInteraction) == 2){
    actualInteractionValues <- generateActualInteractionX2_FixEff(labelsInInteraction,
                                                               l_categoricalVarsInInteraction ,
                                                               data2computeInteraction, 
                                                               l_RefInCategoricalVars)
  }
  return(actualInteractionValues)
}


#' Compute actual interaction values for multiple interaction terms.
#'
#' This function calculates the actual interaction values for multiple interaction terms 
#' using the provided data.
#'
#' @param l_interactionTerm A list of interaction terms in the form of "term1:term2".
#' @param categorical_vars A character vector containing the names of categorical variables in the data.
#' @param dataActual The data frame containing the actual gene expression values and metadata.
#'
#' @return A data frame containing the actual interaction values for each interaction term.
#' @export
#' @examples
#' N_GENES <- 4
#' MIN_REPLICATES <- 3
#' MAX_REPLICATES <- 3
#' init_var <- init_variable(name = "varA", mu = 8, sd = 0.1, level = 3) %>%
#'   init_variable(name = "varB", mu = c(5,-5), NA , level = 2) %>%
#'   init_variable(name = "varC", mu = 1, 3, 3) %>%
#'   add_interaction(between_var = c("varA", "varC"), mu = 5, 0.1)
#' mock_data <- mock_rnaseq(init_var, N_GENES, 
#'                          MIN_REPLICATES, MAX_REPLICATES )
#' data2fit <- prepareData2fit(countMatrix = mock_data$counts, 
#'                              metadata =  mock_data$metadata )
#' results_fit <- fitModelParallel(formula = kij ~ varA + varB + varC + varA:varC,
#'                              data = data2fit, group_by = "geneID",
#'                              family = glmmTMB::nbinom2(link = "log"), n.cores = 1)
#' tidy_tmb <- tidy_tmb(results_fit)
#' fixEff_dataInference  <- subsetFixEffectInferred(tidy_tmb)
#' fixEff_dataActual <- getData2computeActualFixEffect(mock_data$groundTruth$effects)
#' interactionTerm <- fixEff_dataInference$fixed_term$interaction[[1]]
#' categorical_vars <- fixEff_dataActual$categorical_vars
#' dataActual <- fixEff_dataActual$data
#' l_labelsInCategoricalVars <- lapply(dataActual[, categorical_vars], levels)
#' l_interaction <- strsplit(interactionTerm, split = ":")[[1]]
#' l_categoricalVarsInInteraction <- lapply(l_interaction,
#'                                          function(label) findAttribute(label, 
#'                                          l_labelsInCategoricalVars)) %>% 
#'                                          unlist()
#' data_prepared <- prepareData2computeInteraction(categorical_vars, 
#'                    l_categoricalVarsInInteraction, dataActual)
#' # Compute actual interaction values for multiple interactions
#' actualInteraction <- computeActualInteractionFixEff(interactionTerm, categorical_vars, dataActual)
computeActualInteractionFixEff <- function(l_interactionTerm, categorical_vars, dataActual){

  l_interaction <- strsplit(l_interactionTerm, split = ":")
  l_interactionActualValues <- lapply(l_interaction, function(labelsInInteraction)
                                getActualInteractionFixEff(labelsInInteraction, dataActual, categorical_vars))
  actualInteraction_df <- do.call('rbind', l_interactionActualValues)
  return(actualInteraction_df)
}
