# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand


#' Evaluate Dispersion Comparison
#'
#' Compares dispersion values between two data frames containing dispersion information.
#'
#' @param TMB_dispersion_df A data frame containing dispersion values from TMB.
#' @param DESEQ_dispersion_df A data frame containing dispersion values from DESeq2.
#' @param color2use vector of color use for points coloration
#'
#' @return A list containing a dispersion plot and a data frame with dispersion comparison.
#' @importFrom ggplot2 scale_color_manual
#' @export
#'
#' @examples
#' \dontrun{
#' disp_comparison <- evaluateDispersion(TMB_dispersion_df, DESEQ_dispersion_df, "red")
#' plot_dispersion <- disp_comparison$disp_plot
#' comparison_df <- disp_comparison$data
#' }
evaluateDispersion <- function(TMB_dispersion_df, DESEQ_dispersion_df, color2use) {
  disp_comparison_dtf <- rbind(TMB_dispersion_df, DESEQ_dispersion_df)
  disp_plot <- dispersion_plot(disp_comparison_dtf, col = "from", pch = "from") + ggplot2::scale_color_manual(values = color2use)
  return(list(disp_plot = disp_plot, data = disp_comparison_dtf))
}


#' Get Dispersion Comparison
#'
#' Compares inferred dispersion values with actual dispersion values.
#'
#' @param inferred_dispersion A data frame containing inferred dispersion values.
#' @param actual_dispersion A numeric vector containing actual dispersion values.
#'
#' @return A data frame comparing actual and inferred dispersion values.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' dispersion_comparison <- getDispersionComparison(inferred_disp, actual_disp)
#' print(dispersion_comparison)
#' }
getDispersionComparison <- function(inferred_dispersion, actual_dispersion) {
  actual_disp <- data.frame(actual_dispersion = actual_dispersion)
  actual_disp$geneID <- rownames(actual_disp)
  rownames(actual_disp) <- NULL
  disp_comparison <- join_dtf(actual_disp, inferred_dispersion, "geneID", "geneID")
  return(disp_comparison)
}


#' Extract DESeq2 Dispersion Values
#'
#' Extracts inferred dispersion values from a DESeq2 wrapped object.
#'
#' @param deseq_wrapped A DESeq2 wrapped object containing dispersion values.
#'
#' @return A data frame containing inferred dispersion values.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' dispersion_df <- extractDESeqDispersion(deseq2_object)
#' print(dispersion_df)
#' }
extractDESeqDispersion <- function(deseq_wrapped) {
  inferred_dispersion <- data.frame(inferred_dispersion = deseq_wrapped$dispersion)
  inferred_dispersion$geneID <- rownames(inferred_dispersion)
  rownames(inferred_dispersion) <- NULL
  return(inferred_dispersion)
}


#' Extract TMB Dispersion Values
#'
#' Extracts inferred dispersion values from a TMB result object.
#'
#' @param l_tmb A TMB result object containing dispersion values.
#'
#' @return A data frame containing inferred dispersion values.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' dispersion_df <- extractTMBDispersion(tmb_result)
#' print(dispersion_df)
#' }
extractTMBDispersion <- function(l_tmb) {
  glanceRes <- glance_tmb(l_tmb)
  inferred_dispersion <- data.frame(inferred_dispersion = glanceRes$dispersion)
  inferred_dispersion$geneID <- rownames(glanceRes)
  rownames(inferred_dispersion) <- NULL
  return(inferred_dispersion)
}



#' Dispersion Evaluation Plot
#'
#' Creates a scatter plot to evaluate the dispersion values between actual and inferred dispersions.
#'
#' @param eval_dispersion A data frame containing actual and inferred dispersion values.
#' @param ... Additional arguments to be passed to the ggplot2::aes function.
#' @importFrom ggplot2 ggplot geom_point aes geom_abline theme_bw ggtitle scale_x_log10 scale_y_log10
#' @return A ggplot2 scatter plot.
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' disp_plot <- dispersion_plot(disp_comparison_dtf, col = "from")
#' print(disp_plot)
#' }
dispersion_plot <- function(eval_dispersion, ...) {

  args <- lapply(list(...), function(x) if (!is.null(x)) ggplot2::sym(x))

  p <- ggplot2::ggplot(eval_dispersion) +
    ggplot2::geom_point(ggplot2::aes(x = actual_dispersion, y = inferred_dispersion, !!!args), size = 3, alpha = 0.6) +
    ggplot2::geom_abline(intercept = 0, slope = 1, lty = 3, col = 'red', linewidth = 1) +
    ggplot2::theme_bw() +
    ggplot2::ggtitle("Dispersion evaluation") +
    ggplot2::scale_x_log10() +
    ggplot2::scale_y_log10()

  return(p)
}



