# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand


#' Export the Analysis Report to a File
#'
#' This function generates an analysis report by arranging and combining various plots
#' and tables, and then exports the report to a specified file.
#'
#' @param report_file Path to the file where the report will be exported.
#' @param table_settings A table containing settings and parameters used in the analysis.
#' @param roc_curve A plot displaying the Receiver Operating Characteristic (ROC) curve.
#' @param dispersion_plot A plot displaying the dispersion values.
#' @param id_plot A plot displaying unique identifiers.
#' @param counts_plot A plot displaying the gene counts.
#'
#'
#' @importFrom gridExtra arrangeGrob grid.arrange
#' @importFrom ggplot2 ggsave
#'
#'
#' @return report
#' @export
exportReportFile <- function(report_file, table_settings, roc_curve, dispersion_plot, id_plot, counts_plot){

  middle_part  <- gridExtra::arrangeGrob(counts_plot, dispersion_plot, heights = c(1, 1.5))
  left_part  <- gridExtra::arrangeGrob(table_settings, roc_curve ,heights = c(1, 1.5))
  p2export <- gridExtra::grid.arrange(left_part, middle_part, id_plot ,ncol = 3, widths = c(1,1,2))

  if (!is.null(report_file)) ggplot2::ggsave(report_file, p2export, height = 10, width = 15)

  return(p2export)
}


#' Generate a Formatted Table as a Grid Graphics Object
#'
#' This function generates a formatted table using the provided data frame and returns
#' it as a grid graphics object.
#'
#' @param df The data frame to be converted into a formatted table.
#'
#' @return A grid graphics object representing the formatted table.
#' @export
#' @importFrom ggplot2 unit
#' @importFrom gridExtra tableGrob ttheme_minimal
#' @examples
#' # Create a sample data frame
#' sample_data <- data.frame(
#'   Name = c("Alice", "Bob", "Charlie"),
#'   Age = c(25, 30, 28)
#' )
#'
#' # Generate the formatted table
#' table_grob <- getGrobTable(sample_data)
getGrobTable <- function(df){
  theme_custom <- gridExtra::ttheme_minimal(
    core=list(bg_params = list(fill = c("#F8F9F9", "#E5E8E8"), col=NA)),
    colhead=list(fg_params=list(col="white", fontface=4L), bg_params = list(fill = "#5D6D7E", col=NA)),
    base_size = 15)
  grob_df <- gridExtra::tableGrob(df, rows=NULL, theme = theme_custom, widths = ggplot2::unit(x = c(0.4,0.3), "npc" ) )
  return(grob_df)
}


#' Generate a simulation report
#'
#' This function generates a simulation report containing various plots and evaluation metrics.
#'
#' @param mock_obj A list containing simulation data and ground truth.
#' @param list_tmb A list of model results.
#' @param coeff_threshold A threshold for comparing estimates.
#' @param alt_hypothesis The alternative hypothesis for comparisons ("greater", "less", "greaterAbs").
#' @param report_file File name to save the generated report. If NULL, the report will not be exported.
#' @param dds_obj a DESeq2 object
#' @importFrom ggplot2 aes geom_point geom_abline facet_wrap theme_bw ggtitle
#' @return A list containing settings, plots, and evaluation results.
#' @export
simulationReport <- function(mock_obj, list_tmb = NULL,
                             coeff_threshold = 0, alt_hypothesis = "greaterAbs", report_file = NULL, dds_obj = NULL ){

  #-- init 
  TMB_comparison_df <- data.frame()
  DESEQ_comparison_df <- data.frame()
  DESEQ_dispersion_df <- data.frame()
  TMB_dispersion_df <- data.frame()
  
  # -- build data from list_tmb
  if (!is.null(list_tmb)){
      tidyRes  <- tidy_results(list_tmb, coeff_threshold, alt_hypothesis)
      formula_used <- list_tmb[[1]]$modelInfo$allForm$formula
      TMB_comparison_df <- compareInferenceToExpected(tidyRes, mock_obj$groundTruth$effects, formula_used)
      TMB_comparison_df <- getLabelExpected(TMB_comparison_df, coeff_threshold, alt_hypothesis)
      TMB_comparison_df$from <- "HTRfit"
      tmb_disp_inferred <- extractTMBDispersion(list_tmb)
      TMB_dispersion_df <- getDispersionComparison(tmb_disp_inferred, mock_data$groundTruth$gene_dispersion)
      TMB_dispersion_df$from <- 'HTRfit'
  }
  
  if (!is.null(dds_obj)){
      deseq2_wrapped <- wrapper_DESeq2(dds, coeff_threshold, alt_hypothesis)
      DESEQ_comparison_df <- inferenceToExpected_withFixedEff(deseq2_wrapped$fixEff, mock_obj$groundTruth$effects)
      DESEQ_comparison_df <- getLabelExpected(DESEQ_comparison_df, coeff_threshold, alt_hypothesis)
      DESEQ_comparison_df$from <- "DESeq2"
      DESEQ_comparison_df$component <- NA
      DESEQ_comparison_df$group <- NA
      DESEQ_disp_inferred <- extractDESeqDispersion(deseq2_wrapped)
      DESEQ_dispersion_df <- getDispersionComparison(DESEQ_disp_inferred , mock_data$groundTruth$gene_dispersion)
      DESEQ_dispersion_df$from <- 'DESeq2'
  }
  
  comparison_df <- rbind( DESEQ_comparison_df, TMB_comparison_df )
  
  
  #color2use <- c("#D2B4DE", "#A2D9CE")
  color2use <- c("#500472", "#79cbb8")
  color2use <- color2use[c(!is.null(dds_obj), !is.null(list_tmb))]

  # -- plotting
  roc_curve <- roc_plot(comparison_df, col = "from" ) + ggplot2::scale_color_manual(values = color2use)
  id_plot <- identity_plot(comparison_df, col = "from", pch = "from") + ggplot2::scale_color_manual(values = color2use)
  #metrics_plot <- metrics_plot(list_tmb)
  evalDisp <- evaluateDispersion(TMB_dispersion_df, DESEQ_dispersion_df, color2use )
  dispersion_plot <- evalDisp$disp_plot
  counts_plot <- counts_plot(mock_obj)
  
  # -- export report
  df_settings <- mock_obj$settings
  grobTableSettings <- getGrobTable(df_settings)
  exportReportFile(report_file, grobTableSettings, roc_curve, dispersion_plot, id_plot, counts_plot)

  # -- return
  ret <- list(settings = df_settings, roc_plot = roc_curve,
              dispersionEvaluation =  evalDisp, identity_plot = id_plot, counts_plot = counts_plot, data = comparison_df)
  return(ret)
}

