# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand



#' Update GLMNB models in parallel.
#'
#' This function fits GLMNB models in parallel using multiple cores, allowing for faster computation.
#'
#' @param formula Formula for the GLMNB model.
#' @param l_tmb List of GLMNB objects.
#' @param n.cores Number of cores to use for parallel processing. If NULL, the function will use all available cores.
#' @param log_file File path for the log output.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return A list of updated GLMNB models.
#'
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' results <- updateParallel(new_formula, fitted_models, n.cores = 1)
updateParallel <- function(formula, l_tmb, n.cores = NULL, log_file = "log.txt", ...) {
    
    isValidInput2fit(l_tmb[[1]]$frame, formula)
  
    is_fullrank(l_tmb[[1]]$frame, formula)
    
    # Fit models update in parallel and capture the results
    results <- .parallel_update(formula, l_tmb, n.cores, log_file, ...)
    return(results)
}


#' Internal function to fit GLMNB models in parallel.
#'
#' This function is used internally by \code{\link{updateParallel}} to fit GLMNB models in parallel.
#'
#' @param formula Formula for the GLMNB model.
#' @param l_tmb List of GLMNB objects.
#' @param n.cores Number of cores to use for parallel processing.
#' @param log_file File path for the log output.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return A list of updated GLMNB models.
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' results <- .parallel_update(new_formula, fitted_models, n.cores = 1)
.parallel_update <- function(formula, l_tmb, n.cores = NULL, log_file = "log.txt",  ...) {
  if (is.null(n.cores)) n.cores <- parallel::detectCores()
  clust <- parallel::makeCluster(n.cores, outfile = log_file)
  #l_geneID <- attributes(l_tmb)$names
  parallel::clusterExport(clust, c("launchUpdate", "fitUpdate"),  envir=environment())
  updated_res <- parallel::parLapply(clust, X = l_tmb, fun = launchUpdate , formula = formula, ...)
  parallel::stopCluster(clust)
  #closeAllConnections()
  return(updated_res)
}


#' Fit and update a GLMNB model.
#'
#' This function fits and updates a GLMNB model using the provided formula.
#'
#' @param glmnb_obj A GLMNB object to be updated.
#' @param formula Formula for the updated GLMNB model.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return An updated GLMNB model.
#'
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' updated_model <- fitUpdate(fitted_models[[1]], new_formula)
fitUpdate <- function(glmnb_obj, formula , ...){
  data = glmnb_obj$frame
  resUpdt <- stats::update(glmnb_obj, formula, ...)
  resUpdt$frame <- data
  ## family in ... => avoid error in future update
  additional_args <- list(...)
  familyArgs <- additional_args[['family']]
  if (!is.null(familyArgs)) resUpdt$call$family <- familyArgs
  ## control in ... => avoid error in future update
  controlArgs <- additional_args[['control']]
  if (!is.null(controlArgs)) resUpdt$call$control <- controlArgs
  return(resUpdt)
}


#' Launch the update process for a GLMNB model.
#'
#' This function launches the update process for a GLMNB model, capturing and handling warnings and errors.
#'
#' @param glmnb_obj A GLMNB object to be updated.
#' @param formula Formula for the updated GLMNB model.
#' @param ... Additional arguments to be passed to the glmmTMB::glmmTMB function.
#' @export
#' @return An updated GLMNB model or NULL if an error occurs.
#'
#' @examples
#' data(iris)
#' groups <- unique(iris$Species)
#' group_by <- "Species"
#' formula <- Sepal.Length ~ Sepal.Width + Petal.Length
#' fitted_models <- fitModelParallel(formula, iris, group_by, n.cores = 1)
#' new_formula <- Sepal.Length ~ Sepal.Width 
#' updated_model <- launchUpdate(fitted_models[[1]], new_formula)
launchUpdate <- function(glmnb_obj, formula,  ...) {
  group = deparse(substitute(glmnb_obj))
  tryCatch(
    expr = {
      withCallingHandlers(
        fitUpdate(glmnb_obj, formula, ...),
        warning = function(w) {
          message(paste(Sys.time(), "warning for group", group ,":", conditionMessage(w)))
          invokeRestart("muffleWarning")
        })
    },
    error = function(e) {
    message(paste(Sys.time(), "error for group", group,":", conditionMessage(e)))
    return(NULL)
    }
  )
}

